
import sys
import os
import color

from color import colorize

verbose_val = False


def verbose(enable):
    global verbose_val
    if enable is True:
        verbose_val = True
    else:
        verbose_val = False


def _print(str, eol=True, output=sys.stderr):
    print >>output, str,
    if eol:
        print >>output
    else:
        output.flush()


def die(str=None, output=sys.stderr):
    if str is not None:
        _print(color.red(" * ") + str, True, output)
    sys.exit(1)


def warn(str, eol=True, output=sys.stderr):
    _print(color.yellow(" * ") + str, eol, output)


def error(str, eol=True, output=sys.stderr):
    _print(color.red(" * ") + str, eol, output)


def vinfo(str, eol=True, output=sys.stderr):
    global verbose_val
    if verbose_val:
        _print(color.green(" * ") + str, eol, output)


def info(str, eol=True, output=sys.stderr):
    _print(color.green(" * ") + str, eol, output)


def raw_print(str, eol=True, output=sys.stderr):
	_print(str, eol, output)


def is_verbose():
    return verbose_val


def get_term_size():
    """
    Get the number of lines and columns of the tty that is connected to
    stderr.  Returns a tuple of (lines, columns) or (-1, -1) if an error
    occurs. The curses module is used if available, otherwise the output of
    `stty size` is parsed.
    """
    if not sys.stderr.isatty():
        return -1, -1
    try:
        import curses
        try:
            curses.setupterm(term=os.environ.get("TERM", "unknown"),
                fd=sys.stderr.fileno())
            return curses.tigetnum('lines'), curses.tigetnum('cols')
        except curses.error:
            pass
    except ImportError:
        pass
    try:
        import commands
        st, out = commands.getstatusoutput('stty size')
        if st == os.EX_OK:
            out = out.split()
            if len(out) == 2:
                return int(out[0]), int(out[1])
    except:
        pass
    return -1, -1


class EOutput:
    """
    Performs fancy terminal formatting for status and informational messages.

    The provided methods produce identical terminal output to the eponymous
    functions in the shell script C{/sbin/functions.sh} and also accept
    identical parameters.

    This is not currently a drop-in replacement however, as the output-related
    functions in C{/sbin/functions.sh} are oriented for use mainly by system
    init scripts and ebuilds and their output can be customized via certain
    C{RC_*} environment variables (see C{/etc/conf.d/rc}). B{EOutput} is not
    customizable in this manner since it's intended for more general uses.
    Likewise, no logging is provided.

    @ivar quiet: Specifies if output should be silenced.
    @type quiet: BooleanType
    @ivar term_columns: Width of terminal in characters. Defaults to the value
        specified by the shell's C{COLUMNS} variable, else to the queried tty
        size, else to C{80}.
    @type term_columns: IntType
    """

    def __init__(self):
        self.__last_e_cmd = ""
        self.__last_e_len = 0
        self.quiet = False
        lines, columns = get_term_size()
        self.out = sys.stderr
        if columns <= 0:
            columns = 80
        # Adjust columns so that end works properly on a standard BSD console.
        if os.environ.get("TERM") in ("cons25", "dumb"):
            columns = columns - 1
        self.term_columns = columns

    def __print(self, msg=None, eol=True):
        if msg:
            print >>self.out, msg,

        if eol:
            print >>self.out
        else:
            self.out.flush()

    def __end(self, caller, errno, msg):
        if errno == 0:
            status_brackets = colorize("BRACKET", "[ ") + \
                colorize("GOOD", "ok") + colorize("BRACKET", " ]")
        else:
            status_brackets = colorize("BRACKET", "[ ") + \
                colorize("BAD", "!!") + colorize("BRACKET", " ]")
            if msg:
                if caller == "end":
                    self.error(msg[0])
                elif caller == "wend":
                    self.warn(msg[0])
        if self.__last_e_cmd != "begin":
            self.__last_e_len = 0
        self.__print("%*s%s" % ((self.term_columns - self.__last_e_len - 6),
            "", status_brackets))

    def begin(self, msg):
        """
        Shows a message indicating the start of a process.

        @param msg: A very brief (shorter than one line) description of the
            starting process.
        @type msg: StringType
        """
        msg += " ..."
        if not self.quiet:
            self.infon(msg)
            self.__last_e_len = len(msg) + 4
            self.__last_e_cmd = "begin"

    def end(self, errno, *msg):
        """
        Indicates the completion of a process, optionally displaying a message
        via L{error} if the process's exit status isn't C{0}.

        @param errno: A standard UNIX C{errno} code returned by processes upon
            exit.
        @type errno: IntType
        @param msg: I{(optional)} An error message, typically a standard UNIX
            error string corresponding to C{errno}.
        @type msg: StringType
        """
        if not self.quiet:
            self.__end("end", errno, msg)
            self.__last_e_cmd = "end"

    def error(self, msg):
        """
        Shows an error message.

        @param msg: A very brief (shorter than one line) error message.
        @type msg: StringType
        """
        if self.__last_e_cmd == "begin":
            self.__print()
        self.__print(colorize("BAD", " * ") + msg)
        self.__last_e_cmd = "error"

    def info(self, msg):
        """
        Shows an informative message terminated with a newline.

        @param msg: A very brief (shorter than one line) informative message.
        @type msg: StringType
        """
        if not self.quiet:
            if self.__last_e_cmd == "begin":
                self.__print()
            self.__print(colorize("GOOD", " * ") + msg)
            self.__last_e_cmd = "info"

    def infon(self, msg):
        """
        Shows an informative message terminated without a newline.

        @param msg: A very brief (shorter than one line) informative message.
        @type msg: StringType
        """
        if not self.quiet:
            if self.__last_e_cmd == "begin":
                self.__print()
            self.__print(colorize("GOOD", " * ") + msg, False)
            self.__last_e_cmd = "infon"

    def warn(self, msg):
        """
        Shows a warning message.

        @param msg: A very brief (shorter than one line) warning message.
        @type msg: StringType
        """
        if not self.quiet:
            if self.__last_e_cmd == "begin":
                self.__print()
            self.__print(colorize("WARN", " * ") + msg)
            self.__last_e_cmd = "warn"

    def wend(self, errno, *msg):
        """
        Indicates the completion of a process, optionally displaying a message
        via L{warn} if the process's exit status isn't C{0}.

        @param errno: A standard UNIX C{errno} code returned by processes upon
            exit.
        @type errno: IntType
        @param msg: I{(optional)} A warning message, typically a standard UNIX
            error string corresponding to C{errno}.
        @type msg: StringType
        """
        if not self.quiet:
            self.__end("wend", errno, msg)
            self.__last_e_cmd = "wend"

eout = EOutput()

# vim:set noet ts=4 sts=4 sw=4 :
