import os, subprocess, re
from output import is_verbose
from output import vinfo
from output import info
from output import warn
from gittool import git_cmd

try:
    import ConfigParser as configparser
except:
    import configparser


## @brief repo object manipulator
##
class Module():
    def __init__(self, uri, path, params=None):
        self.params = {} if params is None else params
        self.__path = path
        if not uri:
                try:
                    p = git_cmd(["remote", "get-url", "origin"], \
                                stdout=subprocess.PIPE, cwd=self.__path)
                    uri = p.stdout.strip()
                except Exception as e:
                    return None
        self.__uri = uri

    def is_valid_uri(self):
        try:
            vinfo("git ls-remote {}".format(self.uri()))
            git_cmd(["ls-remote", self.uri()], display=True)
            return True
        except Exception as e:
            return False

    ## @brief represent the repo uri and extra info as a string
    def info(self):
        ret = self.__uri
        for k, v in self.params.items():
            if v:
                ret += ';%s=%s' % (k, v)
            else:
                ret += ';%s' % k
        return ret

    def status_str(self):
        path = self.path()
        if path.endswith("/"):
            path = path[:-1]
        if '/' in path:
            path = path.rsplit('/', 1)[1]
        if self.rev_is_branch_head(self.rev()):
            ret = "%-20s Branch: %-20s Revision: <%s> (HEAD)" % (path, self.branch(), self.rev_short())
        else:
            ret = "%-20s Branch: %-20s Revision: <%s>" % (path, self.branch(), self.rev_short())
        return ret

    def __err(self, msg, e=None):
        if is_verbose() and e:
            msg += '\n' + str(e)
        raise Exception(msg)

    def update(self):
        if not os.path.exists(self.__path):
            try:
                print "Cloning ..."
                git_cmd(["clone", self.uri(), self.__path], display=True,
                    cwd=os.getcwd())

            except Exception as e:
                self.__err('Failed to clone repo: %s' % self.uri(), e)
        else:
            try:
                remote = self.params.get('remote', 'origin')
                git_cmd(["remote", "update", remote], display=True, cwd=self.__path)
            except Exception as e:
                self.__err('Failed to update repo: %s' % self.uri(), e)

    ## @brief git rev-parse command
    ##
    ## @param[in] rev : the rev to rev-parse
    ## @return the parsed rev (full hash), None on error
    def rev_parse(self, rev, params=None):
        args = ["rev-parse"]
        if params:
            args.extend(params)
        args.append(rev)
        try:
            p = git_cmd(args, stdout=subprocess.PIPE,
                cwd=self.__path)
            return p.stdout.strip()
        except:
            return None

    ## @brief check that the given ref exists
    def rev_exists(self, rev):
        try:
            p = git_cmd(["cat-file", "-t", rev], stdout=subprocess.PIPE,
                cwd=self.__path)
            return p.stdout.strip() == "commit" or p.stdout.strip() == "tag"
        except:
            # also check if ref exists remotely
            try:
                p = git_cmd(["cat-file", "-t", "origin/" + rev], stdout=subprocess.PIPE,
                    cwd=self.__path)
                return p.stdout.strip() == "commit" or p.stdout.strip() == "tag"
            except:
                return False
            return False

    def rev_date(self, rev):
        try:
            p = git_cmd(["log", "-1", "--format=%cd", "--date=short", rev], stdout=subprocess.PIPE,
                cwd=self.__path)
            return p.stdout.strip()
        except:
            return None

    def rev_is_branch_head(self, rev):
        try:
            p = git_cmd(["for-each-ref", "refs/heads/", "refs/remotes/"], stdout=subprocess.PIPE, cwd=self.__path)
            for line in p.stdout.split(os.linesep):
                sha1, kind, name = line.split()
                name = name.rsplit('/', 1)[1]
                if sha1 != rev and name != rev:
                    continue
                return name
            return None
        except:
            return None

    def status(self, params=None):
        args = ["status"]
        if params:
            args.extend(params)
        p = git_cmd(args, stdout=subprocess.PIPE, cwd=self.__path)
        return p.stdout

    def count(self):
        p = git_cmd(["log", "--pretty=format:''"], stdout=subprocess.PIPE, cwd=self.__path)
        return p.stdout.count("\n")

    def stats(self, from_rev, to_rev="HEAD"):
        p = git_cmd(["diff", "--stat", "%s..%s" % (from_rev, to_rev)],
                stdout=subprocess.PIPE, cwd=self.__path)
        return p.stdout

    def checkout(self, pull=False):
        rev = self.params.get('rev')
        remote = self.params.get('remote', 'origin')
        if rev:
            # Checking out a specific rev
            if not self.rev_exists(rev):
                self.update()
            if not self.rev_exists(rev):
                self.__err('Could not find git rev: %s in repo %s' %
                    (rev, self.__path))
            branch = self.rev_is_branch_head(rev)
            if branch:
                # if rev is local or remote branch head, checkout branch instead of rev
                git_cmd(["checkout", branch], display=True, cwd=self.__path)
                # if checked out branch head does not match rev, or pull was requested, try to merge remote branch
                if self.rev_parse(rev) != self.rev_parse('HEAD') or pull:
                    git_cmd(["merge", "--ff-only", "%s/%s" % (remote, branch)], display=True, cwd=self.__path)
                # if checked out branch still does not match (e.g. because local branch has unpushed commits)
                # then check out specific rev
                if self.rev_parse(rev) != self.rev_parse('HEAD'):
                    git_cmd(["checkout", rev], display=True, cwd=self.__path)
            else:
                git_cmd(["checkout", rev], display=True, cwd=self.__path)
        else:
            branch = self.params.get('branch', 'master')
            self.update()
            try:
                try:
                    git_cmd(["checkout", branch], display=True, cwd=self.__path)
                except:
                    git_cmd(["checkout", "-b", branch, "%s/%s" %
                        (remote, branch)], display=True, cwd=self.__path)
                else:
                    git_cmd(["merge", "--ff-only", "%s/%s" % (remote, branch)], display=True, cwd=self.__path)
            except Exception as e:
                self.__err('Failed to checkout %s branch' % branch, e)
        return self.rev_parse('HEAD')

    def setRev(self, rev):
        self.params['rev'] = rev
        branch = self.rev_is_branch_head(rev)
        if branch:
            self.params['branch'] = branch

    def rev(self):
        return self.params.get('rev', None)

    def rev_short(self):
        rev = self.params.get('rev', None)
        return self.rev_parse(rev, params=['--short'])

    def branch(self):
        return self.params.get('branch', 'master')

    def uri(self):
        return self.__uri.format(**os.environ)

    def path(self):
        return self.__path

    def __repr__(self):
        return self.info()


def getGitServer():
    p = git_cmd(['config', 'remote.origin.url'])
    uri = p.stdout.strip()
    if re.match('^[a-z]*://', uri): # http://server/path or (ssh, git ...)
        clone_uri = '/'.join(uri.split('/')[:3])
        git_uri = 'git@' + clone_uri.split('://')[1]
    else: # git@server:path scheme
        clone_uri = uri.split(':', 1)[0]
        git_uri = clone_uri
    return [clone_uri, git_uri]

def getCoreModule():
    return Module(getGitServer()[0], os.getcwd())

def loadBuildModule(f, b, remote_repo=None, offline=False):
    if not os.path.exists(f):
        raise Exception('No such file %s' % f)

    config = configparser.ConfigParser()
    config.read(f)
    os.environ['GIT_SERVER'] = getGitServer()[0]
    os.environ['GIT_SERVER_SSH'] = getGitServer()[1]

    if not b:
        b = config.get("build-repos", "default")

    # Try with an existing local build repository first
    m = Module(None, b)
    if m and (offline or m.is_valid_uri()):
        info("Using build repo URI {}".format(m.uri()))
        if remote_repo and remote_repo not in m.uri():
            warn("Git remote of local build repo doesn't match specified " \
                 "remote repo name")
        return m, b

    # In case the build repository doesn't exist, find the remote location
    if not remote_repo:
        remote_repo = b

    if remote_repo.startswith("build-exceet-"):
        uri_repos = config.get("build-repos", "exceet")
    elif remote_repo.startswith("build-ktn"):
        uri_repos = config.get("build-repos", "ktn")
    elif remote_repo.startswith("build-stm32mp"):
        uri_repos = config.get("build-repos", "stm32mp")
    else:
        uri_repos = config.get("build-repos", "customer")

    try:
        uri_search = config.get("build-repos", "uri_search_list", "").split()
    except Exception as e:
        uri_search = []
        
    uri_list = [uri_repos] + uri_search
    vinfo("URI list is {}".format(uri_list))
    for buildRepoUri in uri_list:
        buildRepoUri = buildRepoUri + remote_repo + ".git"

        # If HTTP is used, replace the colon
        # after the server address with a slash
        if buildRepoUri.startswith("{GIT_SERVER}") and \
           os.environ['GIT_SERVER'].startswith("http"):
            buildRepoUri = buildRepoUri.replace("}:", "}/")

        m = Module(buildRepoUri, b)
        if offline or m.is_valid_uri():
            # REMARK: With offline option the first repos url is always used
            info("Using build repo URI {}".format(m.uri()))
            return m, b
        else:
            vinfo("Invalid build repo URI {}, continue ...".format(m.uri()))

    raise Exception("No git repository {} found in any of the following " \
                    "URIs:\n{}".format(remote_repo, "\n".join(uri_list)))


## @brief load all repo informations
def loadModules(f, b=None):
    if not os.path.exists(f):
        raise Exception('No such file %s' % f)

    config = configparser.ConfigParser()
    config.read(f)
    os.environ['GIT_SERVER'] = getGitServer()[0]
    os.environ['GIT_SERVER_SSH'] = getGitServer()[1]

    modules = {}
    for name, uri in config.items("modules"):
        uri = uri.split(';')
        if uri[0].startswith("{GIT_SERVER}") and \
          os.environ['GIT_SERVER'].startswith("http"):
            uri[0] = uri[0].replace("}:", "}/")
        if ("{GIT_SERVER}" in uri[0] or \
	  "{GIT_SERVER_SSH}" in uri[0]) and \
	  not uri[0].endswith(".git"):
	    uri[0] += ".git"
        params = {}
        for p in uri[1:]:
            if '=' in p:
                k, v = p.split('=', 1)
                params[k] = v
            else:
                params[k] = None
        uri = uri[0]
        modules[name] = Module(uri, name, params)
    return modules


def saveModules(f, modules, b):
    config = configparser.ConfigParser(allow_no_value=True)
    config.read(f)

    for m in modules.values():
        config.set("modules", m.path(), m.info())
    hdl = open(f + '.tmp', 'w')
    config.write(hdl)
    hdl.close()

    os.unlink(f)
    os.rename(f + '.tmp', f)

